def children_range(id_e, tree_lv, tree_ch, root_id):
    y = tree_ch+1
    lv = tree_lv
    # Найдем уровень элемента
    for i in range(1, tree_lv):
        if ((id_e-root_id) % (y**(tree_lv-i))) == 0:
            lv = i
            break
    if lv == tree_lv:
        return[]
    # Теперь вернем диапазон возможных детей (первого и последнего)
    return [id_e+y**(tree_lv-lv-1), id_e-1+((y**(tree_lv-lv-1))*(tree_ch+1))]


def parents_list(id_e, tree_lv, tree_ch, root_id):
    pl = []
    y = tree_ch+1
    for i in range(1, tree_lv):
        x = y**(tree_lv-i)
        e = x * ((id_e-root_id) // x)
        if e != (id_e-root_id):
            pl.append(e+root_id)

    return pl



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.db import migrations


def load_stores_from_sql():
    import os
    import treensl
    ini_sql = open(os.path.join(treensl.__path__[0],
                                'sql_scripts/ini_postgres.sql'
                                ), 'r', newline='', encoding='utf-8'
                   ).read()
    return ini_sql


class Migration(migrations.Migration):

    dependencies = [

    ]

    operations = [
        migrations.RunSQL(load_stores_from_sql()),
    ]



# -*- coding: utf-8 -*-
from django.db import models

# Этот скрипт необходимо выполнять во всех иерархических таблицах-наследниках
# путем добавления в миграции вашего приложения migrations.RunSQL().
# Пример в файле myapp/migranions/0001_initial.py
POSTGRESQL_CONNECT_TABLE = '''-- тестировал в БД PostgreSQL 9.1 и выше

-- Добавляем поле-массив которое хранит "дырки"
ALTER TABLE  {0} ADD COLUMN holes integer[];

-- Начальный элемент дерева
INSERT INTO {0} ({1}) VALUES {2};

-- Запишем Имя таблицы и Размерность нового дерева
INSERT INTO tree_size (table_name, number_of_levels, number_of_children)
               VALUES ('{0}', {3}, {4});

-- Теперь можно подключить триггерные функции
CREATE TRIGGER after_upd
  AFTER UPDATE
  ON {0}
  FOR EACH ROW
  EXECUTE PROCEDURE {5};

CREATE TRIGGER before_del
  BEFORE DELETE
  ON {0}
  FOR EACH ROW
  EXECUTE PROCEDURE {6};

CREATE TRIGGER before_new
  BEFORE INSERT
  ON {0}
  FOR EACH ROW
  EXECUTE PROCEDURE {7};
'''


class TreeAbstract(models.Model):

    lvl = models.IntegerField(blank=True, null=False)
    created_children = models.IntegerField(blank=True, null=False)
    removed_children = models.IntegerField(blank=True, null=False)

    # Актуальное количество детей можно получить так:
    # self.created_children - self.removed_children

    class Meta:
        abstract = True

    # Подстановка значений в POSTGRESQL_CONNECT_TABLE
    def sql_end(self):

        # В определении модели, поле 'parent_id',названо как 'parent'.
        # Но, так как это поле ссылка на 'id', Django в БД его пишет
        # как 'parent_id'.

        # Создадим списки полей модели и их значений для первой
        # строки (для начального элемента, корня дерева).
        # Начальный элемент в таблице, хранящей дерево, ссылается сам на себя.
        list_fields = ['id', 'parent_id', 'lvl', 'created_children',
                       'removed_children']

        list_values = [self.ROOT_ID, self.ROOT_ID, 0, 0, 0]

        # Наследник модели может иметь свои дополнительные поля,
        # ...и они могут быть NOT NULL.
        # Необходимо вставить начальный элемент дерева, с учетом этого
        # То есть, заполнить такие поля каким-либо значением
        # (0 - подходит для полей всех типов в Postgres)
        for field in self._meta.fields:
            if (field.name not in list_fields) and (field.name != 'parent'):
                list_fields.append(field.name)
                list_values.append(0)

        # Триггерные функции для работы с id
        p5 = 'treensl_after_upd()'
        p6 = 'treensl_before_del()'
        p7 = 'treensl_before_new()'

        return POSTGRESQL_CONNECT_TABLE.format(self._meta.db_table,
                                               ", ".join(list_fields,),
                                               tuple(list_values),
                                               self.LEVELS,
                                               self.CHILDREN,
                                               p5, p6, p7
                                               )


class Tree32Abstract(TreeAbstract):
    # Размерность дерева по умолчанию для ключей типа int32.
    # Можно переопределить в потомках.
    # (некоторые допустимые значаения: 3/1623, 4/255, 5/83, 6/39)
    LEVELS = 5
    CHILDREN = 83

    # Начальный элемент взят минимально допустимый для int32.
    ROOT_ID = -2147483648

    id = models.IntegerField(primary_key=True)
    parent = models.ForeignKey('self')

    class Meta:
        abstract = True


class Tree64Abstract(TreeAbstract):
    # Размерность дерева по умолчанию для ключей типа int64.
    # Можно переопределить в потомках.
    # (некоторые допустимые значаения: 3/2642243, 4/65535, 5/7129,
    #  6/1623, 7/563, 8/255, 9/137, 10/83)
    LEVELS = 6
    CHILDREN = 1623

    # Начальный элемент взят минимально допустимый для int64.
    ROOT_ID = -9223372036854775808

    id = models.BigIntegerField(primary_key=True)
    parent = models.ForeignKey('self')

    class Meta:
        abstract = True
